"""
This file will be invoked by python when called with th `-m` options:

```
python -m skhep_testdata cms_hep_2012_tutorial/data.root
```
"""

from __future__ import annotations

import argparse

from . import data_path, download_all, known_files


def main():
    # type: () -> None
    parser = argparse.ArgumentParser(
        prog="skhep_testdata",
        description="Expand a testing dataset path to a full path,"
        " downloading things if a remote file is request.",
    )

    parser.add_argument("--all", action="store_true", help="Download all files")
    parser.add_argument("--list", action="store_true", help="List all files")
    parser.add_argument(
        "--dir",
        type=str,
        default=None,
        help="Path to store in (DEFAULT: ~/.local/skhepdata)",
    )
    parser.add_argument("file_path", help="Path to expand", nargs="*")
    args = parser.parse_args()

    if args.list:
        print(*sorted(known_files), sep="\n")

    if args.all:
        download_all(args.dir)

    for p in args.file_path:
        path = data_path(p, args.dir)
        print(path)


if __name__ == "__main__":
    main()



from __future__ import annotations

import tempfile
import zipfile
from importlib import resources
from pathlib import Path

import requests

from . import data, remote_files

__all__ = ("data_path", "download_all", "known_files")


DIR = Path(__file__).parent.resolve()

baseurl = "https://raw.githubusercontent.com/scikit-hep/scikit-hep-testdata/main/src/skhep_testdata/data/"
zipurl = "https://github.com/scikit-hep/scikit-hep-testdata/zipball/main"


with resources.as_file(resources.files(data) / "file_list.txt") as fp, fp.open() as f:
    known_files = {n.strip() for n in f if n.strip()}


def _cache_path(cache_dir: str | None = None) -> Path:
    if cache_dir is None:
        skhepdir = Path.home() / ".local" / "skhepdata"
        skhepdir.mkdir(exist_ok=True, parents=True)
        return skhepdir

    return Path(cache_dir)


def data_path(
    filename: str, raise_missing: bool = True, cache_dir: str | None = None
) -> str:
    if remote_files.is_known_remote(filename):
        return remote_files.remote_file(filename, raise_missing=raise_missing)

    if filename not in known_files and raise_missing:
        raise FileNotFoundError(filename)

    filepath = DIR / "data" / filename

    if not filepath.exists() and filename in known_files:
        cached_path = _cache_path(cache_dir) / filename

        if not cached_path.exists():
            # Currently get from main branch
            # Could locally cache if not fixable, or always cache locally
            # Could verify exists first
            # Download all not implemented
            response = requests.get(baseurl + filename)
            response.raise_for_status()
            with cached_path.open("wb") as f:
                f.write(response.content)
        return str(cached_path)

    return str(filepath)


def download_all(cache_dir: str | None = None) -> None:
    local_dir = _cache_path(cache_dir)

    with tempfile.TemporaryFile() as f:
        with requests.get(zipurl, stream=True) as r:
            r.raise_for_status()
            for chunk in r.iter_content(chunk_size=8192):
                f.write(chunk)

        with zipfile.ZipFile(f) as z:
            for n in z.namelist():
                if "src/skhep_testdata/data/" in n and not n.endswith(".py"):
                    z.extract(n, str(local_dir / str(n.split("/")[-1])))



from __future__ import annotations

import errno
import logging
import os
import tarfile
from importlib import resources
from typing import ClassVar
from urllib.request import urlretrieve

import yaml

_default_data_dir = os.path.realpath(os.path.dirname(__file__))


class RemoteDatasetList:
    _all_files: ClassVar[dict[str, dict[str, str]]] = {}

    @classmethod
    def get_config_for_file(cls, filename: str) -> dict[str, str]:
        cls.load_remote_configs()
        config = cls._all_files.get(filename, None)

        if not config:
            msg = f"Unknown remote file: {filename}"
            raise RuntimeError(msg)

        return config.copy()

    @classmethod
    def load_remote_configs(cls, file_to_load: str | None = None) -> None:
        if cls._all_files and file_to_load is None:
            return

        if file_to_load is None:
            dataset_yml = resources.files("skhep_testdata").joinpath(
                "remote_datasets.yml"
            )
            with dataset_yml.open() as infile:
                datasets = yaml.load(infile, Loader=yaml.SafeLoader)
        else:
            with open(file_to_load) as infile:
                datasets = yaml.load(infile, Loader=yaml.SafeLoader)

        for dataset, config in datasets.items():
            files = config["files"]
            if isinstance(files, list):
                files = {f: f for f in files}
                config["files"] = files
            config["dataset_name"] = dataset
            for filename in files:
                scoped_name = os.path.join(dataset, filename)
                cls._all_files[scoped_name] = config

    @classmethod
    def is_known(cls, filename: str) -> bool:
        cls.load_remote_configs()
        return filename in cls._all_files


def make_all_dirs(path: str) -> None:
    try:
        os.makedirs(path)
    except OSError as exc:
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise


def fetch_remote_dataset(
    dataset_name: str, files: dict[str, str], url: str, data_dir: str
) -> None:
    dataset_dir = os.path.join(data_dir, dataset_name)

    writefile = os.path.join(dataset_dir, os.path.basename(url))
    if os.path.exists(writefile):
        return

    make_all_dirs(dataset_dir)
    logging.warning("Downloading %s", url)
    urlretrieve(url, writefile)

    if tarfile.is_tarfile(writefile):
        logging.warning("Extracting %s", writefile)
        with tarfile.open(writefile) as tar:
            members = [tar.getmember(f) for f in files.values()]
            tar.extractall(dataset_dir, members)

        for outfile, infile in files.items():
            full_in = os.path.join(dataset_dir, infile)
            full_out = os.path.join(dataset_dir, outfile)
            os.rename(full_in, full_out)

    if not os.path.exists(writefile):
        msg = "Problem obtaining remote dataset : %s"
        raise RuntimeError(msg % dataset_name)


def is_known_remote(filename: str) -> bool:
    return RemoteDatasetList.is_known(filename)


def remote_file(
    filename: str, data_dir: str = _default_data_dir, raise_missing: bool = False
) -> str:
    config = RemoteDatasetList.get_config_for_file(filename)
    if not config and raise_missing:
        msg = f"Unknown {filename} cannot be found"
        raise RuntimeError(msg)

    path = os.path.join(data_dir, filename)
    if not os.path.isfile(path):
        config["data_dir"] = data_dir
        fetch_remote_dataset(**config)  # type: ignore[arg-type]

    if not os.path.isfile(path) and raise_missing:
        msg = f"{filename} cannot be found"
        raise RuntimeError(msg)

    return path



from __future__ import annotations

from .local_files import data_path, download_all, known_files
from .version import version as __version__

__all__ = ("__version__", "data_path", "download_all", "known_files")


def __dir__() -> tuple[str, ...]:  # pragma: no cover
    return __all__
